// Copyright(c) 2018, Intel Corporation
//
// Redistribution  and  use  in source  and  binary  forms,  with  or  without
// modification, are permitted provided that the following conditions are met:
//
// * Redistributions of  source code  must retain the  above copyright notice,
//   this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
// * Neither the name  of Intel Corporation  nor the names of its contributors
//   may be used to  endorse or promote  products derived  from this  software
//   without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO,  THE
// IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT  SHALL THE COPYRIGHT OWNER  OR CONTRIBUTORS BE
// LIABLE  FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR
// CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT LIMITED  TO,  PROCUREMENT  OF
// SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE,  DATA, OR PROFITS;  OR BUSINESS
// INTERRUPTION)  HOWEVER CAUSED  AND ON ANY THEORY  OF LIABILITY,  WHETHER IN
// CONTRACT,  STRICT LIABILITY,  OR TORT  (INCLUDING NEGLIGENCE  OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

/**
 * @file userclk.h
 * @brief Functions for setting and get afu user clock
*/

#ifndef __FPGA_AFU_USER_CLOCK_H__
#define __FPGA_AFU_USER_CLOCK_H__

#include <opae/types.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * set afu user clock high and low
 * @param[in]  handle       Handle to previously opened accelerator resource.
 * @param[in]  high_clk     AFU High user clock frequency in MHz.
 * @param[in]  low_clk      AFU Low user clock frequency in MHz.
 * @param[in]  flags        Flags Reserved.
 *
.*@returns FPGA_OK on success. FPGA_INVALID_PARAM if invalid parameters were provided, or
 * if the parameter combination is not valid. FPGA_EXCEPTION if an internal
 * exception occurred while trying to access the handle.
 */
fpga_result fpgaSetUserClock(fpga_handle handle,
				uint64_t high_clk, uint64_t low_clk, int flags);

/**
 * Get afu user clock high and low
 * @param[in]   handle       Handle to previously opened accelerator resource.
 * @param[out]  high_clk     AFU High user clock frequency in MHz.
 * @param[out]  low_clk      AFU Low user clock frequency in MHz.
 * @param[in]   flags        Flags  Reserved.
 *
 .*@returns FPGA_OK on success. FPGA_INVALID_PARAM if invalid parameters were provided, or
 * if the parameter combination is not valid. FPGA_EXCEPTION if an internal
 * exception occurred while trying to access the handle.
 */
fpga_result fpgaGetUserClock(fpga_handle handle,
				uint64_t *high_clk, uint64_t *low_clk, int flags);

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

#endif // __FPGA_AFU_USER_CLOCK_H__
